package Onis::Plugins::Conversations;

use strict;
use warnings;

use Onis::Config qw/get_config/;
use Onis::Html qw/get_filehandle/;
use Onis::Language qw/translate/;
use Onis::Data::Core qw#register_plugin get_main_nick get_print_name nick_to_ident ident_to_name#;
use Onis::Data::Persistent qw#init#;

our $DATA;

our @H_IMAGES = qw#dark-theme/h-red.png dark-theme/h-blue.png dark-theme/h-yellow.png dark-theme/h-green.png#;
our $BAR_WIDTH  = 100;

if (get_config ('horizontal_images'))
{
	my @tmp = get_config ('horizontal_images');
	my $i;
	
	if (scalar (@tmp) != 4)
	{
		print STDERR $/, __FILE__, ": The number of horizontal images is not four. The output might look weird.", $/;
	}

	for ($i = 0; $i < 4; $i++)
	{
		if (!defined ($tmp[$i]))
		{
			next;
		}

		$H_IMAGES[$i] = $tmp[$i];
	}
}
if (get_config ('bar_width'))
{
	my $tmp = get_config ('bar_width');
	$tmp =~ s/\D//g;
	$BAR_WIDTH = 2 * $tmp if ($tmp >= 10);
}

$DATA = register_plugin ('TEXT', \&add);
$DATA = register_plugin ('OUTPUT', \&output);

my $VERSION = '$Id: Conversations.pm,v 1.5 2004/08/01 13:46:47 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub add
{
	my $data = shift;
	my $text = $data->{'text'};
	my $nick = $data->{'nick'};
	my $ident = $data->{'ident'};

	my $time = int ($data->{'hour'} / 6);

	# <taken from lib/Onis/Plugins/Nicks.pm>
	my @potential_nicks = split (/[^\w\`\~\^\-\|\[\]]+/, $text);
	my $talk_to = '';
	
	for (@potential_nicks)
	{
		if (defined ($DATA->{'nick_cache'}{$_}))
		{
			$talk_to = nick_to_ident ($_);
			last;
		}
	}
	# </taken>
	
	if ($talk_to)
	{
		my $chars = length ($text);

		if (!defined ($DATA->{'byident'}{$ident}{'conversations'}{$talk_to}))
		{
			$DATA->{'byident'}{$ident}{'conversations'}{$talk_to} = [0, 0, 0, 0];
		}

		$DATA->{'byident'}{$ident}{'conversations'}{$talk_to}[$time] += $chars;
	}
	
	if (!defined ($DATA->{'nick_cache'}{$nick}))
	{
		$DATA->{'nick_cache'}{$nick} = 1;
	}
	else
	{
		$DATA->{'nick_cache'}{$nick}++;
	}
}

sub calculate
{
	my @retval = ();
	my $retval_data = {};
	my @retval_data = ();
	my @names = grep
	{
		defined ($DATA->{'byname'}{$_}{'conversations'})
	} keys (%{$DATA->{'byname'}});
	
	my $max = 10;
	if (get_config ('plugin_max'))
	{
		my $tmp = get_config ('plugin_max');
		$tmp =~ s/\D//g;

		$max = $tmp if ($tmp);
	}

	for (@names)
	{
		my $name0 = $_;
		my $this_data = {};

		next unless (defined ($name0) and $name0);

		for (keys %{$DATA->{'byname'}{$name0}{'conversations'}})
		{
			my $ident1 = $_;
			my $name1 = ident_to_name ($ident1);
			my $name2;
			my $num;

			next unless (defined ($name1) and $name1);
			next unless (defined ($DATA->{'byname'}{$name1}{'lines'}));
			next if ($name0 eq $name1);

			if (!$this_data->{$name1})
			{
				$this_data->{$name1} = $DATA->{'byname'}{$name0}{'conversations'}{$ident1};
			}
			else
			{
				for (my $i = 0; $i < 4; $i++)
				{
					$this_data->{$name1}[$i] += $DATA->{'byname'}{$name0}{'conversations'}{$ident1}[$i];
				}
			}

			if (($DATA->{'byname'}{$name0}{'lines'} >= 100)
					and ($DATA->{'byname'}{$name1}{'lines'} >= 100))
			{
				if ($name0 gt $name1)
				{
					$name2 = $name0;
					$num = '2';
				}
				else
				{
					$name2 = $name1;
					$name1 = $name0;
					$num = '1';
				}

				if (!defined ($retval_data->{$name1}{$name2}))
				{
					$retval_data->{$name1}{$name2} =
					{
						total1	=> 0,
						total2	=> 0,
						num1	=> [0, 0, 0, 0],
						num2	=> [0, 0, 0, 0],
						name1	=> $name1,
						name2	=> $name2
					};
				}

				for (0..3)
				{
					$retval_data->{$name1}{$name2}{"total$num"}   += $DATA->{'byname'}{$name0}{'conversations'}{$ident1}[$_];
					$retval_data->{$name1}{$name2}{"num$num"}[$_] += $DATA->{'byname'}{$name0}{'conversations'}{$ident1}[$_];
				}
			}
		}
		
		$DATA->{'byname'}{$name0}{'conversations'} = $this_data;
	}
	
	for (keys (%$retval_data))
	{
		my $name1 = $_;

		for (keys (%{$retval_data->{$name1}}))
		{
			push (@retval_data, $retval_data->{$name1}{$_});
		}
	}

	@retval = (sort
		{
			($b->{'total1'} + $b->{'total2'})
			<=>
			($a->{'total1'} + $a->{'total2'})
		} (@retval_data));

	if (scalar (@retval) > $max)
	{
		return (@retval[0..($max - 1)]);
	}
	else
	{
		return (@retval);
	}
}

sub output
{
	my @data = calculate ();

	return (undef) unless (@data);

	my $fh = get_filehandle ();
	my $title = translate ('Conversation partners');

	my $max_num = 0;
	my $factor = 0;

	my @img = get_config ('horizontal_images');

	for (@data)
	{
		if ($_->{'total1'} > $max_num)
		{
			$max_num = $_->{'total1'};
		}
		if ($_->{'total2'} > $max_num)
		{
			$max_num = $_->{'total2'};
		}
	}

	return (undef) unless ($max_num);
	$factor = $BAR_WIDTH / $max_num;

	
	print $fh <<EOF;
<table class="plugin">
  <tr>
    <th>$title</th>
  </tr>
EOF
	foreach (@data)
	{
		my $d = $_;
		my $name1 = $d->{'name1'};
		my $name2 = $d->{'name2'};

		print $fh <<EOF;
  <tr>
    <td>
      <table style="margin: 0px; padding: 0px; border-spacing: 0px; width: 100%;">
        <tr>
	  <td style="padding-right: 0.3em; text-align: right; width: 50%;" class="nick">$name1</td>
	  <td style="padding-left:  0.3em; text-align: left;  width: 50%;" class="nick">$name2</td>
	</tr>
	<tr>
EOF

		print $fh '	  <td style="text-align: right; width: 50%;" class="bar">';
		for (3, 2, 1, 0)
		{
			my $i = $img[$_];
			my $w = int (0.5 + ($d->{'num1'}[$_] * $factor));
			my $c = '';
			$w ||= 1;

			$w = $w . 'px';

			if    ($_ == 3) { $c = qq# class="first"#; }
			elsif ($_ == 0) { $c = qq# class="last"#;  }

			print $fh qq#<img src="$i" style="height: 4px; width: $w;"$c alt="" />#;
		}

		print $fh qq#</td>\n\t  <td style="text-align: left;   width: 50%;" class="bar">#;

		for (0, 1, 2, 3)
		{
			my $i = $img[$_];
			my $w = int (0.5 + ($d->{'num2'}[$_] * $factor));
			my $c = '';
			$w ||= 1;

			$w = $w . 'px';

			if    ($_ == 0) { $c = qq# class="first"#; }
			elsif ($_ == 3) { $c = qq# class="last"#;  }

			print $fh qq#<img src="$i" style="height: 4px; width: $w;"$c alt=""/>#;
		}
		print $fh "</td>\n";
		print $fh <<EOF;
	  </tr>
	</table>
      </td>
    </tr>
EOF
	}

	print $fh "</table>\n\n";
}
