package Onis::Plugins::Urls;

use strict;
use warnings;

use Onis::Config qw/get_config/;
use Onis::Html qw/html_escape get_filehandle/;
use Onis::Language qw/translate/;
use Onis::Data::Core qw#register_plugin#;
use Onis::Data::Persistent qw#init#;

our $DATA = init ('$DATA', 'hash');

register_plugin ('TEXT', \&add);
register_plugin ('ACTION', \&add);
register_plugin ('TOPIC', \&add);
register_plugin ('OUTPUT', \&output);

my $VERSION = '$Id: Urls.pm,v 1.4 2003/12/02 14:42:32 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub add
{
	my $data = shift;
	my $text = $data->{'text'};
	
	while ($text =~ m#(?:(?:ftp|https?)://|www\.)[\w\.-]+\.[A-Za-z]{2,4}(?::\d+)?(?:/[\w\d\.\%\/\-\~]*(?:\?[\+\w\&\%\=]+)?)?(?=\W|$)#ig)
	{
		my $match = $&;

		if ($match =~ m/^www/) { $match = 'http://' . $match; }
		if ($match !~ m#://[^/]+/#) { $match .= '/'; }
		
		if (defined $DATA->{$match})
		{
			$DATA->{$match}{'count'}++;
		}
		else
		{
			$DATA->{$match}{'count'} = 1;
		}

		$DATA->{$match}{'usedby'} = $data->{'nick'};
	}
}

sub calculate
{
	my $retval = [];
	
	my $max = 10;
	if (get_config ('plugin_max'))
	{
		my $tmp = get_config ('plugin_max');
		$tmp =~ s/\D//g;

		$max = $tmp if ($tmp);
	}
	
	my @urls = sort
	{
		$DATA->{$b}{'count'} <=> $DATA->{$a}{'count'}
	} (keys %$DATA);

	
	if (scalar (@urls) > $max) { splice (@urls, $max); }

	for (@urls)
	{
		my $url = $_;
		my $usedby = $DATA->{$url}{'usedby'};
		my $count = $DATA->{$url}{'count'};
		
		push (@$retval, [$url, $count, $usedby]);
	}

	return ($retval);
}

sub output
{
	my $data = calculate ();

	return (undef) unless (@$data);

	my $fh = get_filehandle ();

	my $url = translate ('URL');
	my $times = translate ('Times used');
	my $last = translate ('Last used by');
	
	print $fh <<EOF;
<table class="plugin">
  <tr>
    <td class="invis">&nbsp;</td>
    <th>$url</th>
    <th>$times</th>
    <th>$last</th>
  </tr>
EOF
	my $i = 0;
	foreach (@$data)
	{
		$i++;
		my ($url, $count, $usedby) = @$_;

		$url = html_escape ($url);
		
		print $fh "  <tr>\n",
		qq#    <td class="numeration">$i</td>\n#,
		qq#    <td>$url</td>\n#,
		qq#    <td>$count</td>\n#,
		qq#    <td>$usedby</td>\n#,
		qq#  </tr>\n#;
	}

	print $fh "</table>\n\n";
}
