package Onis::Plugins::Words;

use strict;
use warnings;

use Onis::Config qw/get_config/;
use Onis::Html qw/html_escape get_filehandle/;
use Onis::Language qw/translate/;
use Onis::Data::Core qw#all_nicks register_plugin#;
use Onis::Data::Persistent qw#init#;

our $DATA = init ('$DATA', 'hash');

register_plugin ('TEXT', \&add);
register_plugin ('ACTION', \&add);
register_plugin ('OUTPUT', \&output);

our $MIN_LENGTH = 5;
if (get_config ('ignore_words'))
{
	my $tmp = get_config ('ignore_words');
	$tmp =~ s/\D//g;

	$MIN_LENGTH = $tmp if ($tmp);
}

my $VERSION = '$Id: Words.pm,v 1.4 2003/12/02 14:42:32 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub add
{
	my $data = shift;
	my $text = $data->{'text'};
	my $nick = $data->{'nick'};
	my $words = $data->{'words'};
	
	for (@$words)
	{
		my $this_word = lc ($_);
		
		# check length..
		next if (length ($this_word) < $MIN_LENGTH);

		$DATA->{$this_word}{'count'}++;
		$DATA->{$this_word}{'usedby'} = $nick;
	}
}

sub calculate
{
	my $retval = [];

	my $max = 10;
	if (get_config ('plugin_max'))
	{
		my $tmp = get_config ('plugin_max');
		$tmp =~ s/\D//g;

		$max = $tmp if ($tmp);
	}
	
	my @nicks = all_nicks ();
	my @words = sort
	{
		$DATA->{$b}{'count'} <=> $DATA->{$a}{'count'}
	} (keys (%$DATA));

	my $count = 0;
	for (@words)
	{
		my $word = $_;
		if (grep { $_ eq $word } (@nicks))
		{
			next;
		}

		my $num  = $DATA->{$word}{'count'};
		my $nick = $DATA->{$word}{'usedby'};

		push (@$retval, [$word, $num, $nick]);
		$count++;

		if ($count == $max) { last; }
	}
	
	return ($retval);
}

sub output
{
	my $data = calculate ();
	return (undef) unless (@$data);

	my $fh = get_filehandle ();
	
	my $word = translate ('Word');
	my $times = translate ('Times used');
	my $last = translate ('Last used by');
	
	print $fh <<EOF;
<table class="plugin">
  <tr>
    <td class="invis">&nbsp;</td>
    <th>$word</th>
    <th>$times</th>
    <th>$last</th>
  </tr>
EOF

	my $i = 0;
	for (@$data)
	{
		$i++;

		my ($word, $count, $nick) = @$_;
		
		print $fh "  <tr>\n",
		qq#    <td class="numeration">$i</td>\n#,
		qq#    <td>$word</td>\n#,
		qq#    <td>$count</td>\n#,
		qq#    <td class="nick">$nick</td>\n#,
		qq#  </tr>\n#;
	}
	print $fh "</table>\n\n";

	return (1);
}
